@@ -1,10 +1,12 @@
-import sys
 import unittest
 
 from supervisor.tests.base import DummySupervisor
 from supervisor.tests.base import DummyRequest
 from supervisor.tests.base import DummySupervisorRPCNamespace
 
+from supervisor.compat import xmlrpclib
+from supervisor.compat import httplib
+
 class GetFaultDescriptionTests(unittest.TestCase):
     def test_returns_description_for_known_fault(self):
         from supervisor import xmlrpc
@@ -43,7 +45,6 @@ class RPCErrorTests(unittest.TestCase):
 
 class XMLRPCMarshallingTests(unittest.TestCase):
     def test_xmlrpc_marshal(self):
-        import xmlrpclib
         from supervisor import xmlrpc
         data = xmlrpc.xmlrpc_marshal(1)
         self.assertEqual(data, xmlrpclib.dumps((1,), methodresponse=True))
@@ -81,21 +82,15 @@ class XMLRPCHandlerTests(unittest.TestCase):
         supervisor = DummySupervisor()
         subinterfaces = [('supervisor', DummySupervisorRPCNamespace())]
         handler = self._makeOne(supervisor, subinterfaces)
-        import xmlrpclib
         data = xmlrpclib.dumps(('a', 'b'), 'supervisor.noSuchMethod')
         request = DummyRequest('/what/ever', None, None, None)
         handler.continue_request(data, request)
         logdata = supervisor.options.logger.data
-        from supervisor.xmlrpc import loads
-        if loads:
-            expected = 2
-        else:
-            expected = 3
-        self.assertEqual(len(logdata), expected)
+        self.assertEqual(len(logdata), 2)
         self.assertEqual(logdata[-2],
-                         u'XML-RPC method called: supervisor.noSuchMethod()')
+                         'XML-RPC method called: supervisor.noSuchMethod()')
         self.assertEqual(logdata[-1],
-           (u'XML-RPC method supervisor.noSuchMethod() returned fault: '
+           ('XML-RPC method supervisor.noSuchMethod() returned fault: '
             '[1] UNKNOWN_METHOD'))
         self.assertEqual(len(request.producers), 1)
         xml_response = request.producers[0]
@@ -105,21 +100,15 @@ class XMLRPCHandlerTests(unittest.TestCase):
         supervisor = DummySupervisor()
         subinterfaces = [('supervisor', DummySupervisorRPCNamespace())]
         handler = self._makeOne(supervisor, subinterfaces)
-        import xmlrpclib
         data = xmlrpclib.dumps((), 'supervisor.getAPIVersion')
         request = DummyRequest('/what/ever', None, None, None)
         handler.continue_request(data, request)
         logdata = supervisor.options.logger.data
-        from supervisor.xmlrpc import loads
-        if loads:
-            expected = 2
-        else:
-            expected = 3
-        self.assertEqual(len(logdata), expected)
+        self.assertEqual(len(logdata), 2)
         self.assertEqual(logdata[-2],
-               u'XML-RPC method called: supervisor.getAPIVersion()')
+               'XML-RPC method called: supervisor.getAPIVersion()')
         self.assertEqual(logdata[-1],
-            u'XML-RPC method supervisor.getAPIVersion() returned successfully')
+            'XML-RPC method supervisor.getAPIVersion() returned successfully')
         self.assertEqual(len(request.producers), 1)
         xml_response = request.producers[0]
         response = xmlrpclib.loads(xml_response)
@@ -140,19 +129,13 @@ class XMLRPCHandlerTests(unittest.TestCase):
         request = DummyRequest('/what/ever', None, None, None)
         handler.continue_request(data, request)
         logdata = supervisor.options.logger.data
-        from supervisor.xmlrpc import loads
-        if loads:
-            expected = 2
-        else:
-            expected = 3
-        self.assertEqual(len(logdata), expected)
+        self.assertEqual(len(logdata), 2)
         self.assertEqual(logdata[-2],
-               u'XML-RPC method called: supervisor.getAPIVersion()')
+               'XML-RPC method called: supervisor.getAPIVersion()')
         self.assertEqual(logdata[-1],
-            u'XML-RPC method supervisor.getAPIVersion() returned successfully')
+            'XML-RPC method supervisor.getAPIVersion() returned successfully')
         self.assertEqual(len(request.producers), 1)
         xml_response = request.producers[0]
-        import xmlrpclib
         response = xmlrpclib.loads(xml_response)
         from supervisor.rpcinterface import API_VERSION
         self.assertEqual(response[0][0], API_VERSION)
@@ -169,161 +152,100 @@ class XMLRPCHandlerTests(unittest.TestCase):
         request = DummyRequest('/what/ever', None, None, None)
         handler.continue_request(data, request)
         logdata = supervisor.options.logger.data
-        from supervisor.xmlrpc import loads
-        if loads:
-            expected = 1
-        else:
-            expected = 2
-        self.assertEqual(len(logdata), expected)
-        self.assertEqual(logdata[-1],
-               u'XML-RPC request received with no method name')
-        self.assertEqual(len(request.producers), 0)
+        self.assertEqual(len(logdata), 1)
+        self.assertTrue(logdata[0].startswith('XML-RPC request data'))
+        self.assertTrue(repr(data) in logdata[0])
+        self.assertTrue(logdata[0].endswith('is invalid: no method name'))
         self.assertEqual(request._error, 400)
 
-    def test_continue_request_500(self):
+    def test_continue_request_400_if_loads_raises_not_xml(self):
+        supervisor = DummySupervisor()
+        subinterfaces = [('supervisor', DummySupervisorRPCNamespace())]
+        handler = self._makeOne(supervisor, subinterfaces)
+        data = 'this is not an xml-rpc request body'
+        request = DummyRequest('/what/ever', None, None, None)
+        handler.continue_request(data, request)
+        logdata = supervisor.options.logger.data
+        self.assertEqual(len(logdata), 1)
+        self.assertTrue(logdata[0].startswith('XML-RPC request data'))
+        self.assertTrue(repr(data) in logdata[0])
+        self.assertTrue(logdata[0].endswith('is invalid: unmarshallable'))
+        self.assertEqual(request._error, 400)
+
+    def test_continue_request_400_if_loads_raises_weird_xml(self):
+        supervisor = DummySupervisor()
+        subinterfaces = [('supervisor', DummySupervisorRPCNamespace())]
+        handler = self._makeOne(supervisor, subinterfaces)
+        data = '<methodName></methodName><junk></junk>'
+        request = DummyRequest('/what/ever', None, None, None)
+        handler.continue_request(data, request)
+        logdata = supervisor.options.logger.data
+        self.assertEqual(len(logdata), 1)
+        self.assertTrue(logdata[0].startswith('XML-RPC request data'))
+        self.assertTrue(repr(data) in logdata[0])
+        self.assertTrue(logdata[0].endswith('is invalid: unmarshallable'))
+        self.assertEqual(request._error, 400)
+
+    def test_continue_request_500_if_rpcinterface_method_call_raises(self):
         supervisor = DummySupervisor()
         subinterfaces = [('supervisor', DummySupervisorRPCNamespace())]
         handler = self._makeOne(supervisor, subinterfaces)
-        import xmlrpclib
         data = xmlrpclib.dumps((), 'supervisor.raiseError')
         request = DummyRequest('/what/ever', None, None, None)
         handler.continue_request(data, request)
         logdata = supervisor.options.logger.data
-        from supervisor.xmlrpc import loads
-        if loads:
-            expected = 2
-        else:
-            expected = 3
-        self.assertEqual(len(logdata), expected)
-        self.assertEqual(logdata[-2],
-               u'XML-RPC method called: supervisor.raiseError()')
-        self.assertTrue(logdata[-1].startswith('Traceback'))
-        self.assertTrue(logdata[-1].endswith('ValueError: error\n'))
-        self.assertEqual(len(request.producers), 0)
+        self.assertEqual(len(logdata), 2)
+        self.assertEqual(logdata[0],
+               'XML-RPC method called: supervisor.raiseError()')
+        self.assertTrue("unexpected exception" in logdata[1])
+        self.assertTrue(repr(data) in logdata[1])
+        self.assertTrue("Traceback" in logdata[1])
+        self.assertTrue("ValueError: error" in logdata[1])
         self.assertEqual(request._error, 500)
 
-class TraverseTests(unittest.TestCase):
-    def test_underscore(self):
-        from supervisor import xmlrpc
-        self.assertRaises(xmlrpc.RPCError, xmlrpc.traverse, None, '_', None)
+    def test_continue_request_500_if_xmlrpc_dumps_raises(self):
+        supervisor = DummySupervisor()
+        subinterfaces = [('supervisor', DummySupervisorRPCNamespace())]
+        handler = self._makeOne(supervisor, subinterfaces)
+        data = xmlrpclib.dumps((), 'supervisor.getXmlRpcUnmarshallable')
+        request = DummyRequest('/what/ever', None, None, None)
+        handler.continue_request(data, request)
+        logdata = supervisor.options.logger.data
+        self.assertEqual(len(logdata), 3)
+        self.assertEqual(logdata[0],
+               'XML-RPC method called: supervisor.getXmlRpcUnmarshallable()')
+        self.assertEqual(logdata[1],
+               'XML-RPC method supervisor.getXmlRpcUnmarshallable() '
+               'returned successfully')
+        self.assertTrue("unexpected exception" in logdata[2])
+        self.assertTrue(repr(data) in logdata[2])
+        self.assertTrue("Traceback" in logdata[2])
+        self.assertTrue("TypeError: cannot marshal" in logdata[2])
+        self.assertEqual(request._error, 500)
 
-    def test_notfound(self):
-        from supervisor import xmlrpc
-        self.assertRaises(xmlrpc.RPCError, xmlrpc.traverse, None, 'foo', None)
+    def test_continue_request_value_is_function(self):
+        class DummyRPCNamespace(object):
+            def foo(self):
+                def inner(self):
+                    return 1
+                inner.delay = .05
+                return inner
+        supervisor = DummySupervisor()
+        subinterfaces = [('supervisor', DummySupervisorRPCNamespace()),
+                          ('ns1', DummyRPCNamespace())]
+        handler = self._makeOne(supervisor, subinterfaces)
+        data = xmlrpclib.dumps((), 'ns1.foo')
+        request = DummyRequest('/what/ever', None, None, None)
+        handler.continue_request(data, request)
+        logdata = supervisor.options.logger.data
+        self.assertEqual(len(logdata), 2)
+        self.assertEqual(logdata[-2],
+               'XML-RPC method called: ns1.foo()')
+        self.assertEqual(logdata[-1],
+            'XML-RPC method ns1.foo() returned successfully')
+        self.assertEqual(len(request.producers), 0)
+        self.assertEqual(request._done, False)
 
-    def test_badparams(self):
-        from supervisor import xmlrpc
-        self.assertRaises(xmlrpc.RPCError, xmlrpc.traverse, self,
-                          'test_badparams', (1, 2, 3))
-
-    def test_success(self):
-        from supervisor import xmlrpc
-        L = []
-        class Dummy:
-            def foo(self, a):
-                L.append(a)
-        dummy = Dummy()
-        xmlrpc.traverse(dummy, 'foo', [1])
-        self.assertEqual(L, [1])
-
-class SupervisorTransportTests(unittest.TestCase):
-    def _getTargetClass(self):
-        from supervisor.xmlrpc import SupervisorTransport
-        return SupervisorTransport
-
-    def _makeOne(self, *arg, **kw):
-        return self._getTargetClass()(*arg, **kw)
-
-    def test_ctor_unix(self):
-        from supervisor import xmlrpc
-        transport = self._makeOne('user', 'pass', 'unix:///foo/bar')
-        conn = transport._get_connection()
-        self.assertTrue(isinstance(conn, xmlrpc.UnixStreamHTTPConnection))
-        self.assertEqual(conn.host, 'localhost')
-        self.assertEqual(conn.socketfile, '/foo/bar')
-
-    def test__get_connection_http_9001(self):
-        import httplib
-        transport = self._makeOne('user', 'pass', 'http://127.0.0.1:9001/')
-        conn = transport._get_connection()
-        self.assertTrue(isinstance(conn, httplib.HTTPConnection))
-        self.assertEqual(conn.host, '127.0.0.1')
-        self.assertEqual(conn.port, 9001)
-
-    def test__get_connection_http_80(self):
-        import httplib
-        transport = self._makeOne('user', 'pass', 'http://127.0.0.1/')
-        conn = transport._get_connection()
-        self.assertTrue(isinstance(conn, httplib.HTTPConnection))
-        self.assertEqual(conn.host, '127.0.0.1')
-        self.assertEqual(conn.port, 80)
-
-    def test_request_non_200_response(self):
-        import xmlrpclib
-        transport = self._makeOne('user', 'pass', 'http://127.0.0.1/')
-        dummy_conn = DummyConnection(400, '')
-        def getconn():
-            return dummy_conn
-        transport._get_connection = getconn
-        self.assertRaises(xmlrpclib.ProtocolError,
-                          transport.request, 'localhost', '/', '')
-        self.assertEqual(transport.connection, None)
-        self.assertEqual(dummy_conn.closed, True)
-
-    def test_request_400_response(self):
-        import xmlrpclib
-        transport = self._makeOne('user', 'pass', 'http://127.0.0.1/')
-        dummy_conn = DummyConnection(400, '')
-        def getconn():
-            return dummy_conn
-        transport._get_connection = getconn
-        self.assertRaises(xmlrpclib.ProtocolError,
-                          transport.request, 'localhost', '/', '')
-        self.assertEqual(transport.connection, None)
-        self.assertEqual(dummy_conn.closed, True)
-        self.assertEqual(dummy_conn.requestargs[0], 'POST')
-        self.assertEqual(dummy_conn.requestargs[1], '/')
-        self.assertEqual(dummy_conn.requestargs[2], '')
-        self.assertEqual(dummy_conn.requestargs[3]['Content-Length'], '0')
-        self.assertEqual(dummy_conn.requestargs[3]['Content-Type'], 'text/xml')
-        self.assertEqual(dummy_conn.requestargs[3]['Authorization'],
-                         'Basic dXNlcjpwYXNz')
-        self.assertEqual(dummy_conn.requestargs[3]['Accept'], 'text/xml')
-
-    def test_request_200_response(self):
-        transport = self._makeOne('user', 'pass', 'http://127.0.0.1/')
-        response = """<?xml version="1.0"?>
-        <methodResponse>
-        <params>
-        <param>
-        <value><string>South Dakota</string></value>
-        </param>
-        </params>
-        </methodResponse>"""
-        dummy_conn = DummyConnection(200, response)
-        def getconn():
-            return dummy_conn
-        transport._get_connection = getconn
-        result = transport.request('localhost', '/', '')
-        self.assertEqual(transport.connection, dummy_conn)
-        self.assertEqual(dummy_conn.closed, False)
-        self.assertEqual(dummy_conn.requestargs[0], 'POST')
-        self.assertEqual(dummy_conn.requestargs[1], '/')
-        self.assertEqual(dummy_conn.requestargs[2], '')
-        self.assertEqual(dummy_conn.requestargs[3]['Content-Length'], '0')
-        self.assertEqual(dummy_conn.requestargs[3]['Content-Type'], 'text/xml')
-        self.assertEqual(dummy_conn.requestargs[3]['Authorization'],
-                         'Basic dXNlcjpwYXNz')
-        self.assertEqual(dummy_conn.requestargs[3]['Accept'], 'text/xml')
-        self.assertEqual(result, ('South Dakota',))
-
-    def test_works_with_py25(self):
-        instance = self._makeOne('username', 'password', 'http://127.0.0.1')
-        # the test is just to insure that this method can be called; failure
-        # would be an AttributeError for _use_datetime under Python 2.5
-        parser, unmarshaller = instance.getparser() # this uses _use_datetime
-
-class IterparseLoadsTests(unittest.TestCase):
     def test_iterparse_loads_methodcall(self):
         s = """<?xml version="1.0"?>
         <methodCall>
@@ -389,10 +311,10 @@ class IterparseLoadsTests(unittest.TestCase):
         </params>
         </methodCall>
         """
-        from supervisor.xmlrpc import loads
-        if loads is None:
-            return # no cElementTree
-        result = loads(s)
+        supervisor = DummySupervisor()
+        subinterfaces = [('supervisor', DummySupervisorRPCNamespace())]
+        handler = self._makeOne(supervisor, subinterfaces)
+        result = handler.loads(s)
         params, method = result
         import datetime
         self.assertEqual(method, 'examples.getStateName')
@@ -409,6 +331,194 @@ class IterparseLoadsTests(unittest.TestCase):
         self.assertEqual(params[10], [12, 'abc', 'def', 34])
         self.assertEqual(params[11], {'k': [1, {}]})
 
+class TraverseTests(unittest.TestCase):
+    def test_security_disallows_underscore_methods(self):
+        from supervisor import xmlrpc
+        class Root:
+            pass
+        class A:
+            def _danger(self):
+                return True
+        root = Root()
+        root.a = A()
+        self.assertRaises(xmlrpc.RPCError, xmlrpc.traverse,
+            root, 'a._danger', [])
+
+    def test_security_disallows_object_traversal(self):
+        from supervisor import xmlrpc
+        class Root:
+            pass
+        class A:
+            pass
+        class B:
+            def danger(self):
+                return True
+        root = Root()
+        root.a = A()
+        root.a.b = B()
+        self.assertRaises(xmlrpc.RPCError, xmlrpc.traverse,
+            root, 'a.b.danger', [])
+
+    def test_namespace_name_not_found(self):
+        from supervisor import xmlrpc
+        class Root:
+            pass
+        root = Root()
+        self.assertRaises(xmlrpc.RPCError, xmlrpc.traverse,
+            root, 'notfound.hello', None)
+
+    def test_method_name_not_found(self):
+        from supervisor import xmlrpc
+        class Root:
+            pass
+        class A:
+            pass
+        root = Root()
+        root.a = A()
+        self.assertRaises(xmlrpc.RPCError, xmlrpc.traverse,
+            root, 'a.notfound', [])
+
+    def test_method_name_exists_but_is_not_a_method(self):
+        from supervisor import xmlrpc
+        class Root:
+            pass
+        class A:
+            pass
+        class B:
+            pass
+        root = Root()
+        root.a = A()
+        root.a.b = B()
+        self.assertRaises(xmlrpc.RPCError, xmlrpc.traverse,
+            root, 'a.b', [])  # b is not a method
+
+    def test_bad_params(self):
+        from supervisor import xmlrpc
+        class Root:
+            pass
+        class A:
+            def hello(self, name):
+                return "Hello %s" % name
+        root = Root()
+        root.a = A()
+        self.assertRaises(xmlrpc.RPCError, xmlrpc.traverse,
+            root, 'a.hello', ["there", "extra"])  # too many params
+
+    def test_success(self):
+        from supervisor import xmlrpc
+        class Root:
+            pass
+        class A:
+            def hello(self, name):
+                return "Hello %s" % name
+        root = Root()
+        root.a = A()
+        result = xmlrpc.traverse(root, 'a.hello', ["there"])
+        self.assertEqual(result, "Hello there")
+
+class SupervisorTransportTests(unittest.TestCase):
+    def _getTargetClass(self):
+        from supervisor.xmlrpc import SupervisorTransport
+        return SupervisorTransport
+
+    def _makeOne(self, *arg, **kw):
+        return self._getTargetClass()(*arg, **kw)
+
+    def test_ctor_unix(self):
+        from supervisor import xmlrpc
+        transport = self._makeOne('user', 'pass', 'unix:///foo/bar')
+        conn = transport._get_connection()
+        self.assertTrue(isinstance(conn, xmlrpc.UnixStreamHTTPConnection))
+        self.assertEqual(conn.host, 'localhost')
+        self.assertEqual(conn.socketfile, '/foo/bar')
+
+    def test_ctor_unknown(self):
+        self.assertRaises(ValueError,
+            self._makeOne, 'user', 'pass', 'unknown:///foo/bar'
+            )
+
+    def test__get_connection_http_9001(self):
+        transport = self._makeOne('user', 'pass', 'http://127.0.0.1:9001/')
+        conn = transport._get_connection()
+        self.assertTrue(isinstance(conn, httplib.HTTPConnection))
+        self.assertEqual(conn.host, '127.0.0.1')
+        self.assertEqual(conn.port, 9001)
+
+    def test__get_connection_http_80(self):
+        transport = self._makeOne('user', 'pass', 'http://127.0.0.1/')
+        conn = transport._get_connection()
+        self.assertTrue(isinstance(conn, httplib.HTTPConnection))
+        self.assertEqual(conn.host, '127.0.0.1')
+        self.assertEqual(conn.port, 80)
+
+    def test_request_non_200_response(self):
+        transport = self._makeOne('user', 'pass', 'http://127.0.0.1/')
+        dummy_conn = DummyConnection(400, '')
+        def getconn():
+            return dummy_conn
+        transport._get_connection = getconn
+        self.assertRaises(xmlrpclib.ProtocolError,
+                          transport.request, 'localhost', '/', '')
+        self.assertEqual(transport.connection, None)
+        self.assertEqual(dummy_conn.closed, True)
+
+    def test_request_400_response(self):
+        transport = self._makeOne('user', 'pass', 'http://127.0.0.1/')
+        dummy_conn = DummyConnection(400, '')
+        def getconn():
+            return dummy_conn
+        transport._get_connection = getconn
+        self.assertRaises(xmlrpclib.ProtocolError,
+                          transport.request, 'localhost', '/', '')
+        self.assertEqual(transport.connection, None)
+        self.assertEqual(dummy_conn.closed, True)
+        self.assertEqual(dummy_conn.requestargs[0], 'POST')
+        self.assertEqual(dummy_conn.requestargs[1], '/')
+        self.assertEqual(dummy_conn.requestargs[2], b'')
+        self.assertEqual(dummy_conn.requestargs[3]['Content-Length'], '0')
+        self.assertEqual(dummy_conn.requestargs[3]['Content-Type'], 'text/xml')
+        self.assertEqual(dummy_conn.requestargs[3]['Authorization'],
+                         'Basic dXNlcjpwYXNz')
+        self.assertEqual(dummy_conn.requestargs[3]['Accept'], 'text/xml')
+
+    def test_request_200_response(self):
+        transport = self._makeOne('user', 'pass', 'http://127.0.0.1/')
+        response = """<?xml version="1.0"?>
+        <methodResponse>
+        <params>
+        <param>
+        <value><string>South Dakota</string></value>
+        </param>
+        </params>
+        </methodResponse>"""
+        dummy_conn = DummyConnection(200, response)
+        def getconn():
+            return dummy_conn
+        transport._get_connection = getconn
+        result = transport.request('localhost', '/', '')
+        self.assertEqual(transport.connection, dummy_conn)
+        self.assertEqual(dummy_conn.closed, False)
+        self.assertEqual(dummy_conn.requestargs[0], 'POST')
+        self.assertEqual(dummy_conn.requestargs[1], '/')
+        self.assertEqual(dummy_conn.requestargs[2], b'')
+        self.assertEqual(dummy_conn.requestargs[3]['Content-Length'], '0')
+        self.assertEqual(dummy_conn.requestargs[3]['Content-Type'], 'text/xml')
+        self.assertEqual(dummy_conn.requestargs[3]['Authorization'],
+                         'Basic dXNlcjpwYXNz')
+        self.assertEqual(dummy_conn.requestargs[3]['Accept'], 'text/xml')
+        self.assertEqual(result, ('South Dakota',))
+
+    def test_close(self):
+        transport = self._makeOne('user', 'pass', 'http://127.0.0.1/')
+        dummy_conn = DummyConnection(200, '''<?xml version="1.0"?>
+        <methodResponse><params/></methodResponse>''')
+        def getconn():
+            return dummy_conn
+        transport._get_connection = getconn
+        transport.request('localhost', '/', '')
+        transport.close()
+        self.assertTrue(dummy_conn.closed)
+
 class TestDeferredXMLRPCResponse(unittest.TestCase):
     def _getTargetClass(self):
         from supervisor.xmlrpc import DeferredXMLRPCResponse
@@ -480,6 +590,43 @@ class TestDeferredXMLRPCResponse(unittest.TestCase):
         self.assertEqual(src, 'XML-RPC response callback error')
         self.assertTrue("Traceback" in msg)
 
+    def test_getresponse_http_10_with_keepalive(self):
+        inst = self._makeOne()
+        inst.request.version = '1.0'
+        inst.request.header.append('Connection: keep-alive')
+        inst.getresponse('abc')
+        self.assertEqual(len(inst.request.producers), 1)
+        self.assertEqual(inst.request.headers['Connection'], 'Keep-Alive')
+
+    def test_getresponse_http_10_no_keepalive(self):
+        inst = self._makeOne()
+        inst.request.version = '1.0'
+        inst.getresponse('abc')
+        self.assertEqual(len(inst.request.producers), 1)
+        self.assertEqual(inst.request.headers['Connection'], 'close')
+
+    def test_getresponse_http_11_without_close(self):
+        inst = self._makeOne()
+        inst.request.version = '1.1'
+        inst.getresponse('abc')
+        self.assertEqual(len(inst.request.producers), 1)
+        self.assertTrue('Connection' not in inst.request.headers)
+
+    def test_getresponse_http_11_with_close(self):
+        inst = self._makeOne()
+        inst.request.header.append('Connection: close')
+        inst.request.version = '1.1'
+        inst.getresponse('abc')
+        self.assertEqual(len(inst.request.producers), 1)
+        self.assertEqual(inst.request.headers['Connection'], 'close')
+
+    def test_getresponse_http_unknown(self):
+        inst = self._makeOne()
+        inst.request.version = None
+        inst.getresponse('abc')
+        self.assertEqual(len(inst.request.producers), 1)
+        self.assertEqual(inst.request.headers['Connection'], 'close')
+
 class TestSystemNamespaceRPCInterface(unittest.TestCase):
     def _makeOne(self, namespaces=()):
         from supervisor.xmlrpc import SystemNamespaceRPCInterface
@@ -736,11 +883,11 @@ class Test_capped_int(unittest.TestCase):
         self.assertEqual(self._callFUT('42'), 42)
 
     def test_caps_value_below_minint(self):
-        import xmlrpclib
+        from supervisor.compat import xmlrpclib
         self.assertEqual(self._callFUT(xmlrpclib.MININT - 1), xmlrpclib.MININT)
 
     def test_caps_value_above_maxint(self):
-        import xmlrpclib
+        from supervisor.compat import xmlrpclib
         self.assertEqual(self._callFUT(xmlrpclib.MAXINT + 1), xmlrpclib.MAXINT)
 
 
@@ -771,9 +918,3 @@ class DummyConnection:
     def close(self):
         self.closed = True
 
-def test_suite():
-    return unittest.findTestCases(sys.modules[__name__])
-
-if __name__ == '__main__':
-    unittest.main(defaultTest='test_suite')
-
