@@ -2,12 +2,42 @@ import unittest
 import os
 import sys
 
+from supervisor.compat import as_bytes
+
 from supervisor.tests.base import DummyOptions
 from supervisor.tests.base import DummyProcess
 from supervisor.tests.base import DummyPConfig
 from supervisor.tests.base import DummyLogger
 from supervisor.tests.base import DummyEvent
 
+class PDispatcherTests(unittest.TestCase):
+    def setUp(self):
+        from supervisor.events import clear
+        clear()
+
+    def tearDown(self):
+        from supervisor.events import clear
+        clear()
+
+    def _getTargetClass(self):
+        from supervisor.dispatchers import PDispatcher
+        return PDispatcher
+
+    def _makeOne(self, process=None, channel='stdout', fd=0):
+        return self._getTargetClass()(process, channel, fd)
+
+    def test_readable(self):
+        inst = self._makeOne()
+        self.assertRaises(NotImplementedError, inst.readable)
+
+    def test_writable(self):
+        inst = self._makeOne()
+        self.assertRaises(NotImplementedError, inst.writable)
+
+    def test_flush(self):
+        inst = self._makeOne()
+        self.assertEqual(inst.flush(), None)
+
 class POutputDispatcherTests(unittest.TestCase):
     def setUp(self):
         from supervisor.events import clear
@@ -60,24 +90,24 @@ class POutputDispatcherTests(unittest.TestCase):
 
     def test_handle_read_event(self):
         options = DummyOptions()
-        options.readfd_result = 'abc'
+        options.readfd_result = b'abc'
         config = DummyPConfig(options, 'process1', '/bin/process1',
                               stdout_capture_maxbytes=100)
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
         self.assertEqual(dispatcher.handle_read_event(), None)
-        self.assertEqual(dispatcher.output_buffer, 'abc')
+        self.assertEqual(dispatcher.output_buffer, b'abc')
 
     def test_handle_read_event_no_data_closes(self):
         options = DummyOptions()
-        options.readfd_result = ''
+        options.readfd_result = b''
         config = DummyPConfig(options, 'process1', '/bin/process1',
                               stdout_capture_maxbytes=100)
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
         self.assertFalse(dispatcher.closed)
         self.assertEqual(dispatcher.handle_read_event(), None)
-        self.assertEqual(dispatcher.output_buffer, '')
+        self.assertEqual(dispatcher.output_buffer, b'')
         self.assertTrue(dispatcher.closed)
 
     def test_handle_error(self):
@@ -102,7 +132,7 @@ class POutputDispatcherTests(unittest.TestCase):
         process.pid = 4000
         dispatcher = self._makeOne(process)
         dispatcher.capturemode = True
-        dispatcher.capturelog.data = ['hallooo']
+        dispatcher.capturelog.getvalue = lambda: 'hallooo'
         L = []
         def doit(event):
             L.append(event)
@@ -122,8 +152,8 @@ class POutputDispatcherTests(unittest.TestCase):
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
         dispatcher.removelogs()
-        self.assertEqual(dispatcher.mainlog.handlers[0].reopened, True)
-        self.assertEqual(dispatcher.mainlog.handlers[0].removed, True)
+        self.assertEqual(dispatcher.normallog.handlers[0].reopened, True)
+        self.assertEqual(dispatcher.normallog.handlers[0].removed, True)
         self.assertEqual(dispatcher.childlog.handlers[0].reopened, True)
         self.assertEqual(dispatcher.childlog.handlers[0].removed, True)
 
@@ -135,7 +165,7 @@ class POutputDispatcherTests(unittest.TestCase):
         dispatcher = self._makeOne(process)
         dispatcher.reopenlogs()
         self.assertEqual(dispatcher.childlog.handlers[0].reopened, True)
-        self.assertEqual(dispatcher.mainlog.handlers[0].reopened, True)
+        self.assertEqual(dispatcher.normallog.handlers[0].reopened, True)
 
     def test_record_output_log_non_capturemode(self):
         # stdout/stderr goes to the process log and the main log,
@@ -152,7 +182,7 @@ class POutputDispatcherTests(unittest.TestCase):
         self.assertEqual(dispatcher.childlog.data, ['a'])
         self.assertEqual(options.logger.data[0],
              "'process1' stdout output:\na")
-        self.assertEqual(dispatcher.output_buffer, '')
+        self.assertEqual(dispatcher.output_buffer, b'')
 
     def test_record_output_emits_stdout_event_when_enabled(self):
         options = DummyOptions()
@@ -160,7 +190,7 @@ class POutputDispatcherTests(unittest.TestCase):
                               stdout_events_enabled=True)
         process = DummyProcess(config)
         dispatcher = self._makeOne(process, 'stdout')
-        dispatcher.output_buffer = 'hello from stdout'
+        dispatcher.output_buffer = b'hello from stdout'
 
         L = []
         def doit(event):
@@ -172,7 +202,7 @@ class POutputDispatcherTests(unittest.TestCase):
         self.assertEqual(len(L), 1)
         event = L[0]
         self.assertEqual(event.process, process)
-        self.assertEqual(event.data, 'hello from stdout')
+        self.assertEqual(event.data, b'hello from stdout')
 
     def test_record_output_does_not_emit_stdout_event_when_disabled(self):
         options = DummyOptions()
@@ -180,7 +210,7 @@ class POutputDispatcherTests(unittest.TestCase):
                               stdout_events_enabled=False)
         process = DummyProcess(config)
         dispatcher = self._makeOne(process, 'stdout')
-        dispatcher.output_buffer = 'hello from stdout'
+        dispatcher.output_buffer = b'hello from stdout'
 
         L = []
         def doit(event):
@@ -197,7 +227,7 @@ class POutputDispatcherTests(unittest.TestCase):
                               stderr_events_enabled=True)
         process = DummyProcess(config)
         dispatcher = self._makeOne(process, 'stderr')
-        dispatcher.output_buffer = 'hello from stderr'
+        dispatcher.output_buffer = b'hello from stderr'
 
         L = []
         def doit(event):
@@ -209,7 +239,7 @@ class POutputDispatcherTests(unittest.TestCase):
         self.assertEqual(len(L), 1)
         event = L[0]
         self.assertEqual(event.process, process)
-        self.assertEqual(event.data, 'hello from stderr')
+        self.assertEqual(event.data, b'hello from stderr')
 
     def test_record_output_does_not_emit_stderr_event_when_disabled(self):
         options = DummyOptions()
@@ -217,7 +247,7 @@ class POutputDispatcherTests(unittest.TestCase):
                               stderr_events_enabled=False)
         process = DummyProcess(config)
         dispatcher = self._makeOne(process, 'stderr')
-        dispatcher.output_buffer = 'hello from stderr'
+        dispatcher.output_buffer = b'hello from stderr'
 
         L = []
         def doit(event):
@@ -240,10 +270,10 @@ class POutputDispatcherTests(unittest.TestCase):
                               stdout_capture_maxbytes=100)
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
-        dispatcher.output_buffer = 'stdout string longer than a token'
+        dispatcher.output_buffer = b'stdout string longer than a token'
         dispatcher.record_output()
         self.assertEqual(dispatcher.childlog.data,
-                         ['stdout string longer than a token'])
+                         [b'stdout string longer than a token'])
         self.assertEqual(options.logger.data[0],
              "'process1' stdout output:\nstdout string longer than a token")
 
@@ -273,7 +303,7 @@ class POutputDispatcherTests(unittest.TestCase):
         subscribe(ProcessCommunicationEvent, doit)
         BEGIN_TOKEN = ProcessCommunicationEvent.BEGIN_TOKEN
         END_TOKEN = ProcessCommunicationEvent.END_TOKEN
-        data = BEGIN_TOKEN + 'hello' + END_TOKEN
+        data = BEGIN_TOKEN + b'hello' + END_TOKEN
         options = DummyOptions()
         from supervisor.loggers import getLogger
         options.getLogger = getLogger # actually use real logger
@@ -287,8 +317,8 @@ class POutputDispatcherTests(unittest.TestCase):
         try:
             dispatcher.output_buffer = data
             dispatcher.record_output()
-            self.assertEqual(open(logfile, 'r').read(), '')
-            self.assertEqual(dispatcher.output_buffer, '')
+            self.assertEqual(os.path.getsize(logfile), 0)
+            self.assertEqual(len(dispatcher.output_buffer), 0)
             self.assertEqual(len(events), 1)
 
             event = events[0]
@@ -296,7 +326,7 @@ class POutputDispatcherTests(unittest.TestCase):
             self.assertEqual(event.__class__, ProcessCommunicationStdoutEvent)
             self.assertEqual(event.process, process)
             self.assertEqual(event.channel, 'stdout')
-            self.assertEqual(event.data, 'hello')
+            self.assertEqual(event.data, b'hello')
 
         finally:
             try:
@@ -315,16 +345,17 @@ class POutputDispatcherTests(unittest.TestCase):
         subscribe(ProcessCommunicationEvent, doit)
         import string
         # ascii_letters for python 3
-        letters = getattr(string, "letters", string.ascii_letters)
-        digits = string.digits * 4
+        letters = as_bytes(getattr(string, "letters", string.ascii_letters))
+        digits = as_bytes(string.digits) * 4
         BEGIN_TOKEN = ProcessCommunicationEvent.BEGIN_TOKEN
         END_TOKEN = ProcessCommunicationEvent.END_TOKEN
         data = (letters +  BEGIN_TOKEN + digits + END_TOKEN + letters)
 
         # boundaries that split tokens
-        broken = data.split(':')
-        first = broken[0] + ':'
-        second = broken[1] + ':'
+        colon = b':'
+        broken = data.split(colon)
+        first = broken[0] + colon
+        second = broken[1] + colon
         third = broken[2]
 
         options = DummyOptions()
@@ -339,23 +370,26 @@ class POutputDispatcherTests(unittest.TestCase):
         try:
             dispatcher.output_buffer = first
             dispatcher.record_output()
-            [ x.flush() for x in dispatcher.childlog.handlers]
-            self.assertEqual(open(logfile, 'r').read(), letters)
+            [ x.flush() for x in dispatcher.childlog.handlers ]
+            with open(logfile, 'rb') as f:
+                self.assertEqual(f.read(), letters)
             self.assertEqual(dispatcher.output_buffer, first[len(letters):])
             self.assertEqual(len(events), 0)
 
             dispatcher.output_buffer += second
             dispatcher.record_output()
             self.assertEqual(len(events), 0)
-            [ x.flush() for x in dispatcher.childlog.handlers]
-            self.assertEqual(open(logfile, 'r').read(), letters)
+            [ x.flush() for x in dispatcher.childlog.handlers ]
+            with open(logfile, 'rb') as f:
+                self.assertEqual(f.read(), letters)
             self.assertEqual(dispatcher.output_buffer, first[len(letters):])
             self.assertEqual(len(events), 0)
 
             dispatcher.output_buffer += third
             dispatcher.record_output()
-            [ x.flush() for x in dispatcher.childlog.handlers]
-            self.assertEqual(open(logfile, 'r').read(), letters *2)
+            [ x.flush() for x in dispatcher.childlog.handlers ]
+            with open(logfile, 'rb') as f:
+                self.assertEqual(f.read(), letters * 2)
             self.assertEqual(len(events), 1)
             event = events[0]
             from supervisor.events import ProcessCommunicationStdoutEvent
@@ -379,8 +413,8 @@ class POutputDispatcherTests(unittest.TestCase):
                               stdout_logfile='/tmp/foo')
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
-        ansi = '\x1b[34mHello world... this is longer than a token!\x1b[0m'
-        noansi = 'Hello world... this is longer than a token!'
+        ansi = b'\x1b[34mHello world... this is longer than a token!\x1b[0m'
+        noansi = b'Hello world... this is longer than a token!'
 
         dispatcher.output_buffer = ansi
         dispatcher.record_output()
@@ -394,7 +428,7 @@ class POutputDispatcherTests(unittest.TestCase):
         self.assertEqual(len(dispatcher.childlog.data), 2)
         self.assertEqual(dispatcher.childlog.data[1], ansi)
 
-    def test_ctor_nologfiles(self):
+    def test_ctor_no_logfiles(self):
         options = DummyOptions()
         config = DummyPConfig(options, 'process1', '/bin/process1')
         process = DummyProcess(config)
@@ -403,7 +437,7 @@ class POutputDispatcherTests(unittest.TestCase):
         self.assertEqual(dispatcher.channel, 'stdout')
         self.assertEqual(dispatcher.fd, 0)
         self.assertEqual(dispatcher.capturelog, None)
-        self.assertEqual(dispatcher.mainlog, None)
+        self.assertEqual(dispatcher.normallog, None)
         self.assertEqual(dispatcher.childlog, None)
 
     def test_ctor_logfile_only(self):
@@ -416,8 +450,8 @@ class POutputDispatcherTests(unittest.TestCase):
         self.assertEqual(dispatcher.channel, 'stdout')
         self.assertEqual(dispatcher.fd, 0)
         self.assertEqual(dispatcher.capturelog, None)
-        self.assertEqual(dispatcher.mainlog.__class__, DummyLogger)
-        self.assertEqual(dispatcher.childlog, dispatcher.mainlog)
+        self.assertEqual(dispatcher.normallog.__class__, DummyLogger)
+        self.assertEqual(dispatcher.childlog, dispatcher.normallog)
 
     def test_ctor_capturelog_only(self):
         options = DummyOptions()
@@ -428,21 +462,91 @@ class POutputDispatcherTests(unittest.TestCase):
         self.assertEqual(dispatcher.process, process)
         self.assertEqual(dispatcher.channel, 'stdout')
         self.assertEqual(dispatcher.fd, 0)
-        self.assertEqual(dispatcher.capturelog.__class__,DummyLogger)
-        self.assertEqual(dispatcher.mainlog, None)
+        self.assertEqual(dispatcher.capturelog.__class__, DummyLogger)
+        self.assertEqual(dispatcher.normallog, None)
         self.assertEqual(dispatcher.childlog, None)
 
-    def test_ctor_nologs(self):
+    def test_ctor_stdout_logfile_is_empty_string(self):
+        from supervisor.datatypes import logfile_name
         options = DummyOptions()
-        config = DummyPConfig(options, 'process1', '/bin/process1')
+        config = DummyPConfig(options, 'process1', '/bin/process1',
+                              stdout_logfile=logfile_name(''))
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
         self.assertEqual(dispatcher.process, process)
         self.assertEqual(dispatcher.channel, 'stdout')
         self.assertEqual(dispatcher.fd, 0)
-        self.assertEqual(dispatcher.capturelog, None)
-        self.assertEqual(dispatcher.mainlog, None)
-        self.assertEqual(dispatcher.childlog, None)
+        self.assertEqual(dispatcher.normallog, None)
+
+    def test_ctor_stdout_logfile_none_and_stdout_syslog_false(self):
+        from supervisor.datatypes import boolean, logfile_name
+        options = DummyOptions()
+        config = DummyPConfig(options, 'process1', '/bin/process1',
+                              stdout_logfile=logfile_name('NONE'),
+                              stdout_syslog=boolean('false'))
+        process = DummyProcess(config)
+        dispatcher = self._makeOne(process)
+        self.assertEqual(dispatcher.process, process)
+        self.assertEqual(dispatcher.channel, 'stdout')
+        self.assertEqual(dispatcher.fd, 0)
+        self.assertEqual(dispatcher.normallog, None)
+
+    def test_ctor_stdout_logfile_none_and_stdout_syslog_true(self):
+        from supervisor.datatypes import boolean, logfile_name
+        from supervisor.loggers import LevelsByName, SyslogHandler
+        from supervisor.options import ServerOptions
+        options = ServerOptions() # need real options to get a real logger
+        options.loglevel = LevelsByName.TRAC
+        config = DummyPConfig(options, 'process1', '/bin/process1',
+                              stdout_logfile=logfile_name('NONE'),
+                              stdout_syslog=boolean('true'))
+        process = DummyProcess(config)
+        dispatcher = self._makeOne(process)
+        self.assertEqual(dispatcher.process, process)
+        self.assertEqual(dispatcher.channel, 'stdout')
+        self.assertEqual(dispatcher.fd, 0)
+        self.assertEqual(len(dispatcher.normallog.handlers), 1)
+        self.assertEqual(dispatcher.normallog.handlers[0].__class__,
+            SyslogHandler)
+
+    def test_ctor_stdout_logfile_str_and_stdout_syslog_false(self):
+        from supervisor.datatypes import boolean, logfile_name
+        from supervisor.loggers import FileHandler, LevelsByName
+        from supervisor.options import ServerOptions
+        options = ServerOptions() # need real options to get a real logger
+        options.loglevel = LevelsByName.TRAC
+        config = DummyPConfig(options, 'process1', '/bin/process1',
+                              stdout_logfile=logfile_name('/tmp/foo'),
+                              stdout_syslog=boolean('false'))
+        process = DummyProcess(config)
+        dispatcher = self._makeOne(process)
+        self.assertEqual(dispatcher.process, process)
+        self.assertEqual(dispatcher.channel, 'stdout')
+        self.assertEqual(dispatcher.fd, 0)
+        self.assertEqual(len(dispatcher.normallog.handlers), 1)
+        self.assertEqual(dispatcher.normallog.handlers[0].__class__, FileHandler)
+        dispatcher.normallog.close()
+
+    def test_ctor_stdout_logfile_str_and_stdout_syslog_true(self):
+        from supervisor.datatypes import boolean, logfile_name
+        from supervisor.loggers import FileHandler, LevelsByName, SyslogHandler
+        from supervisor.options import ServerOptions
+        options = ServerOptions() # need real options to get a real logger
+        options.loglevel = LevelsByName.TRAC
+        config = DummyPConfig(options, 'process1', '/bin/process1',
+                              stdout_logfile=logfile_name('/tmp/foo'),
+                              stdout_syslog=boolean('true'))
+        process = DummyProcess(config)
+        dispatcher = self._makeOne(process)
+        self.assertEqual(dispatcher.process, process)
+        self.assertEqual(dispatcher.channel, 'stdout')
+        self.assertEqual(dispatcher.fd, 0)
+        self.assertEqual(len(dispatcher.normallog.handlers), 2)
+        self.assertTrue(any(isinstance(h, FileHandler) for h in
+            dispatcher.normallog.handlers))
+        self.assertTrue(any(isinstance(h, SyslogHandler) for h in
+            dispatcher.normallog.handlers))
+        dispatcher.normallog.close()
 
     def test_repr(self):
         options = DummyOptions()
@@ -450,9 +554,9 @@ class POutputDispatcherTests(unittest.TestCase):
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
         drepr = repr(dispatcher)
-        self.assertTrue(drepr.startswith('<POutputDispatcher at'), drepr)
+        self.assertTrue('POutputDispatcher' in drepr)
         self.assertNotEqual(
-            drepr.find('<supervisor.tests.base.DummyProcess instance at'),
+            drepr.find('supervisor.tests.base.DummyProcess'),
             -1)
         self.assertTrue(drepr.endswith('(stdout)>'), drepr)
 
@@ -523,9 +627,9 @@ class PInputDispatcherTests(unittest.TestCase):
         config = DummyPConfig(options, 'test', '/test')
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
-        self.assertEqual(dispatcher.input_buffer, '')
+        self.assertEqual(dispatcher.input_buffer, b'')
         dispatcher.handle_write_event()
-        self.assertEqual(dispatcher.input_buffer, '')
+        self.assertEqual(dispatcher.input_buffer, b'')
         self.assertEqual(options.written, {})
 
     def test_handle_write_event_epipe_raised(self):
@@ -535,9 +639,10 @@ class PInputDispatcherTests(unittest.TestCase):
         dispatcher = self._makeOne(process)
         dispatcher.input_buffer = 'halloooo'
         import errno
-        options.write_error = errno.EPIPE
+        options.write_exception = OSError(errno.EPIPE,
+                                          os.strerror(errno.EPIPE))
         dispatcher.handle_write_event()
-        self.assertEqual(dispatcher.input_buffer, '')
+        self.assertEqual(dispatcher.input_buffer, b'')
         self.assertTrue(options.logger.data[0].startswith(
             'fd 0 closed, stopped monitoring'))
         self.assertTrue(options.logger.data[0].endswith('(stdin)>'))
@@ -549,7 +654,8 @@ class PInputDispatcherTests(unittest.TestCase):
         dispatcher = self._makeOne(process)
         dispatcher.input_buffer = 'halloooo'
         import errno
-        options.write_error = errno.EBADF
+        options.write_exception = OSError(errno.EBADF,
+                                          os.strerror(errno.EBADF))
         self.assertRaises(OSError, dispatcher.handle_write_event)
 
     def test_handle_write_event_over_os_limit(self):
@@ -587,9 +693,9 @@ class PInputDispatcherTests(unittest.TestCase):
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
         drepr = repr(dispatcher)
-        self.assertTrue(drepr.startswith('<PInputDispatcher at'), drepr)
+        self.assertTrue('PInputDispatcher' in drepr)
         self.assertNotEqual(
-            drepr.find('<supervisor.tests.base.DummyProcess instance at'),
+            drepr.find('supervisor.tests.base.DummyProcess'),
             -1)
         self.assertTrue(drepr.endswith('(stdin)>'), drepr)
 
@@ -661,7 +767,7 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         options.readfd_result = dispatcher.READY_FOR_EVENTS_TOKEN
         self.assertEqual(dispatcher.handle_read_event(), None)
         self.assertEqual(process.listener_state, EventListenerStates.READY)
-        self.assertEqual(dispatcher.state_buffer, '')
+        self.assertEqual(dispatcher.state_buffer, b'')
         self.assertEqual(len(dispatcher.childlog.data), 1)
         self.assertEqual(dispatcher.childlog.data[0],
                          dispatcher.READY_FOR_EVENTS_TOKEN)
@@ -673,14 +779,14 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
         self.assertEqual(dispatcher.handle_read_event(), None)
-        self.assertEqual(dispatcher.state_buffer, '')
+        self.assertEqual(dispatcher.state_buffer, b'')
         from supervisor.dispatchers import EventListenerStates
         self.assertEqual(dispatcher.process.listener_state,
                          EventListenerStates.ACKNOWLEDGED)
 
     def test_handle_read_event_logging_nologs(self):
         options = DummyOptions()
-        options.readfd_result = 'supercalifragilisticexpialidocious'
+        options.readfd_result = b'supercalifragilisticexpialidocious'
         config = DummyPConfig(options, 'process1', '/bin/process1')
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
@@ -691,7 +797,7 @@ class PEventListenerDispatcherTests(unittest.TestCase):
 
     def test_handle_read_event_logging_childlog(self):
         options = DummyOptions()
-        options.readfd_result = 'supercalifragilisticexpialidocious'
+        options.readfd_result = b'supercalifragilisticexpialidocious'
         config = DummyPConfig(options, 'process1', '/bin/process1',
                               stdout_logfile='/tmp/foo')
         process = DummyProcess(config)
@@ -699,7 +805,7 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         self.assertEqual(dispatcher.handle_read_event(), None)
         self.assertEqual(len(dispatcher.childlog.data), 1)
         self.assertEqual(dispatcher.childlog.data[0],
-                         'supercalifragilisticexpialidocious')
+                         b'supercalifragilisticexpialidocious')
 
     def test_handle_listener_state_change_from_unknown(self):
         options = DummyOptions()
@@ -708,9 +814,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         from supervisor.dispatchers import EventListenerStates
         dispatcher = self._makeOne(process)
         process.listener_state = EventListenerStates.UNKNOWN
-        dispatcher.state_buffer = 'whatever'
+        dispatcher.state_buffer = b'whatever'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, '')
+        self.assertEqual(dispatcher.state_buffer, b'')
         self.assertEqual(options.logger.data, [])
         self.assertEqual(process.listener_state, EventListenerStates.UNKNOWN)
 
@@ -721,9 +827,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         from supervisor.dispatchers import EventListenerStates
         dispatcher = self._makeOne(process)
         process.listener_state = EventListenerStates.ACKNOWLEDGED
-        dispatcher.state_buffer = 'READY\n'
+        dispatcher.state_buffer = b'READY\n'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, '')
+        self.assertEqual(dispatcher.state_buffer, b'')
         self.assertEqual(options.logger.data[0],
                          'process1: ACKNOWLEDGED -> READY')
         self.assertEqual(process.listener_state, EventListenerStates.READY)
@@ -735,9 +841,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         from supervisor.dispatchers import EventListenerStates
         dispatcher = self._makeOne(process)
         process.listener_state = EventListenerStates.ACKNOWLEDGED
-        dispatcher.state_buffer = 'READY\ngarbage\n'
+        dispatcher.state_buffer = b'READY\ngarbage\n'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, '')
+        self.assertEqual(dispatcher.state_buffer, b'')
         self.assertEqual(options.logger.data[0],
                          'process1: ACKNOWLEDGED -> READY')
         self.assertEqual(options.logger.data[1],
@@ -751,9 +857,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         from supervisor.dispatchers import EventListenerStates
         dispatcher = self._makeOne(process)
         process.listener_state = EventListenerStates.ACKNOWLEDGED
-        dispatcher.state_buffer = 'RE'
+        dispatcher.state_buffer = b'RE'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, 'RE')
+        self.assertEqual(dispatcher.state_buffer, b'RE')
         self.assertEqual(options.logger.data, [])
         self.assertEqual(process.listener_state,
                          EventListenerStates.ACKNOWLEDGED)
@@ -765,9 +871,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         from supervisor.dispatchers import EventListenerStates
         dispatcher = self._makeOne(process)
         process.listener_state = EventListenerStates.ACKNOWLEDGED
-        dispatcher.state_buffer = 'bogus data yo'
+        dispatcher.state_buffer = b'bogus data yo'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, '')
+        self.assertEqual(dispatcher.state_buffer, b'')
         self.assertEqual(options.logger.data[0],
                          'process1: ACKNOWLEDGED -> UNKNOWN')
         self.assertEqual(options.logger.data[1],
@@ -783,9 +889,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         from supervisor.dispatchers import EventListenerStates
         dispatcher = self._makeOne(process)
         process.listener_state = EventListenerStates.READY
-        dispatcher.state_buffer = 'bogus data yo'
+        dispatcher.state_buffer = b'bogus data yo'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, '')
+        self.assertEqual(dispatcher.state_buffer, b'')
         self.assertEqual(options.logger.data[0],
                          'process1: READY -> UNKNOWN')
         self.assertEqual(options.logger.data[1],
@@ -801,9 +907,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         from supervisor.dispatchers import EventListenerStates
         dispatcher = self._makeOne(process)
         process.listener_state = EventListenerStates.BUSY
-        dispatcher.state_buffer = 'bogus data yo'
+        dispatcher.state_buffer = b'bogus data yo'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, 'bogus data yo')
+        self.assertEqual(dispatcher.state_buffer, b'bogus data yo')
         self.assertEqual(process.listener_state, EventListenerStates.BUSY)
 
     def test_handle_listener_state_change_busy_to_acknowledged_procd(self):
@@ -819,9 +925,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         process.group.config = Dummy()
         from supervisor.dispatchers import default_handler
         process.group.config.result_handler = default_handler
-        dispatcher.state_buffer = 'RESULT 2\nOKabc'
+        dispatcher.state_buffer = b'RESULT 2\nOKabc'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, 'abc')
+        self.assertEqual(dispatcher.state_buffer, b'abc')
         self.assertEqual(options.logger.data[0],
                          'process1: event was processed')
         self.assertEqual(options.logger.data[1],
@@ -842,9 +948,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         process.group.config = Dummy()
         from supervisor.dispatchers import default_handler
         process.group.config.result_handler = default_handler
-        dispatcher.state_buffer = 'RESULT 4\nFAILabc'
+        dispatcher.state_buffer = b'RESULT 4\nFAILabc'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, 'abc')
+        self.assertEqual(dispatcher.state_buffer, b'abc')
         self.assertEqual(options.logger.data[0],
                          'process1: event was rejected')
         self.assertEqual(options.logger.data[1],
@@ -867,9 +973,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         process.listener_state = EventListenerStates.BUSY
         current_event = DummyEvent()
         process.event = current_event
-        dispatcher.state_buffer = 'bogus data\n'
+        dispatcher.state_buffer = b'bogus data\n'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, '')
+        self.assertEqual(dispatcher.state_buffer, b'')
         self.assertEqual(options.logger.data[0],
                 "process1: bad result line: 'bogus data'")
         self.assertEqual(options.logger.data[1],
@@ -896,9 +1002,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         process.group.config = Dummy()
         from supervisor.dispatchers import default_handler
         process.group.config.result_handler = default_handler
-        dispatcher.state_buffer = 'RESULT 2\nOKbogus data\n'
+        dispatcher.state_buffer = b'RESULT 2\nOKbogus data\n'
         self.assertEqual(dispatcher.handle_listener_state_change(), None)
-        self.assertEqual(dispatcher.state_buffer, '')
+        self.assertEqual(dispatcher.state_buffer, b'')
         self.assertEqual(options.logger.data[0],
                          'process1: event was processed')
         self.assertEqual(options.logger.data[1],
@@ -1046,8 +1152,8 @@ class PEventListenerDispatcherTests(unittest.TestCase):
                               stdout_logfile='/tmp/foo')
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
-        ansi = '\x1b[34mHello world... this is longer than a token!\x1b[0m'
-        noansi = 'Hello world... this is longer than a token!'
+        ansi = b'\x1b[34mHello world... this is longer than a token!\x1b[0m'
+        noansi = b'Hello world... this is longer than a token!'
 
         options.readfd_result = ansi
         dispatcher.handle_read_event()
@@ -1088,9 +1194,9 @@ class PEventListenerDispatcherTests(unittest.TestCase):
         process = DummyProcess(config)
         dispatcher = self._makeOne(process)
         drepr = repr(dispatcher)
-        self.assertTrue(drepr.startswith('<PEventListenerDispatcher at'), drepr)
+        self.assertTrue('PEventListenerDispatcher' in drepr)
         self.assertNotEqual(
-            drepr.find('<supervisor.tests.base.DummyProcess instance at'),
+            drepr.find('supervisor.tests.base.DummyProcess'),
             -1)
         self.assertTrue(drepr.endswith('(stdout)>'), drepr)
 
@@ -1111,15 +1217,15 @@ class stripEscapeTests(unittest.TestCase):
         return stripEscapes(s)
 
     def test_zero_length_string(self):
-        self.assertEqual(self._callFUT(''), '')
+        self.assertEqual(self._callFUT(b''), b'')
 
     def test_ansi(self):
-        ansi = '\x1b[34mHello world... this is longer than a token!\x1b[0m'
-        noansi = 'Hello world... this is longer than a token!'
+        ansi = b'\x1b[34mHello world... this is longer than a token!\x1b[0m'
+        noansi = b'Hello world... this is longer than a token!'
         self.assertEqual(self._callFUT(ansi), noansi)
 
     def test_noansi(self):
-        noansi = 'Hello world... this is longer than a token!'
+        noansi = b'Hello world... this is longer than a token!'
         self.assertEqual(self._callFUT(noansi), noansi)
 
 def test_suite():
