@@ -522,7 +522,7 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         self.assertEqual(result, None)
         lines = plugin.ctl.stdout.getvalue().split('\n')
         self.assertEqual(len(lines), 12)
-        self.assertEqual(lines[0], 'output line')
+        self.assertEqual(lines[0], 'stdout line')
 
     def test_tail_no_file(self):
         plugin = self._makeOne()
@@ -557,7 +557,7 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         self.assertEqual(result, None)
         lines = plugin.ctl.stdout.getvalue().split('\n')
         self.assertEqual(len(lines), 3)
-        self.assertEqual(lines[0], 'tput line')
+        self.assertEqual(lines[0], 'dout line')
 
     def test_tail_explicit_channel_stdout_nomodifier(self):
         plugin = self._makeOne()
@@ -565,7 +565,7 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         self.assertEqual(result, None)
         lines = plugin.ctl.stdout.getvalue().split('\n')
         self.assertEqual(len(lines), 12)
-        self.assertEqual(lines[0], 'output line')
+        self.assertEqual(lines[0], 'stdout line')
 
     def test_tail_explicit_channel_stderr_nomodifier(self):
         plugin = self._makeOne()
@@ -573,7 +573,7 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         self.assertEqual(result, None)
         lines = plugin.ctl.stdout.getvalue().split('\n')
         self.assertEqual(len(lines), 12)
-        self.assertEqual(lines[0], 'output line')
+        self.assertEqual(lines[0], 'stderr line')
 
     def test_tail_explicit_channel_unrecognized(self):
         plugin = self._makeOne()
@@ -1287,6 +1287,16 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         out = plugin.ctl.stdout.getvalue()
         self.assertTrue("Shut the remote supervisord down" in out)
 
+    def test_shutdown_with_arg_shows_error(self):
+        plugin = self._makeOne()
+        options = plugin.ctl.options
+        result = plugin.do_shutdown('bad')
+        self.assertEqual(result, None)
+        self.assertEqual(options._server.supervisor._shutdown, False)
+        val = plugin.ctl.stdout.getvalue()
+        self.assertTrue(val.startswith('Error: shutdown accepts no arguments'), val)
+        self.assertEqual(plugin.ctl.exitstatus, LSBInitExitStatuses.GENERIC)
+
     def test_shutdown(self):
         plugin = self._makeOne()
         options = plugin.ctl.options
